<?php

namespace Teknisa\MAN\Services;

use Doctrine\ORM;
use Zeedhi\Framework\DTO\Response\Message;
use Teknisa\MAN\Utilities;

class DoacaoCartaoService {
    
    protected $consumidorService;
    protected $environment;
    
    public function __construct(ORM\EntityManager $entityManager, $environment)  {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
    }
    
    public function buscaConsumidor($CDIDCONSUMID, $NRORG){
		$params = array(
			':CDIDCONSUMID' => $CDIDCONSUMID,
			':NRORG'        => $NRORG
		);

		return $this->entityManager->getConnection()->fetchAll(Utilities\Query::BUSCA_CONSUMIDOR_ESPECIFICO, $params);
	}
    
    public function buscaBeneficiosCampanha($CDCAMPANHA, $NRSEQPRODCAMP, $NRORG){
        $params = array(
			'CDCAMPANHA' 	 => $CDCAMPANHA, 
			'NRSEQPRODCAMP'  => $NRSEQPRODCAMP,
			'NRORG'          => $NRORG
		);		

		return $this->entityManager->getConnection()->fetchAll(Utilities\Query::BUSCA_BENEFICIO_CAMPANHA, $params);
    }
    
    public function buscaParamGeralFidelidade($NRORG){
        $params = array(
			'NRORG'          => $NRORG
		);
		return $this->entityManager->getConnection()->fetchAll(Utilities\Query::BUSCA_PARAMGERAL_FIDELIDADE, $params);
	}
    
    public function criaDigitoVerificador($cartao){

    	$digito = 0;
        $soma = 0;
        $temp = $cartao;
        $fator =2;
        //cria numero verificador
        //multiplica cada digito dependendo do resto da divisao dele por 2
        //soma o resultado
        for ($i = strlen($cartao) - 1; $i >= 0; $i--) {
        	$soma = $temp[$i]*$fator;
        //se o resultado tiver mais que 1 digito soma os 2 digitos
	        if($soma>9){
	         	$soma = ((int)$soma/10) + ((int)$soma%10);
	        }
	        $digito = $digito + (int)$soma;
	    //alterna o fator
        	if($fator == 2){
				$fator = 1;
        	} else{
        		$fator = 2;
        	}
        }
        $resto = $digito % 10;
        //se o resto da divisao por 10 for zero retorna zero, senao retorna resto-10
        if ($resto == 0 ){
        	$digito = 0;
        }else{
	    // a diferença de 10
	        $digito = 10 - $resto;
        }
    	//retorna os 10 digitos
    	return $cartao.(int)$digito;
    }
    
    public function criaCartaoDoacao($params){
		return $this->entityManager->getConnection()->executeQuery(Utilities\Query::UPDATE_CARTAO_DOACAO, $params);
	}
	
	public function insereBeneficioDoacao($params){
		return $this->entityManager->getConnection()->executeQuery(Utilities\Query::INSERE_BENEFICIO_DOACAO, $params);
	}
    
    public function saveCartao($params, $response){
        $cont = 0;
        $erro = 0;
        $arrAux = array();
        $arrCad = array('Nenhum Cartão foi Cadastrado');
        try {
	        
	        $beneficiosCampanha = self::buscaBeneficiosCampanha($params['CDCAMPANHA'], $params['NRSEQPRODCAMP'], $params['NRORG']);

	        $beneficiosCompra = array();

	        for ($i = 0; $i < count($beneficiosCampanha); ++$i) {
	        // caso seja um beneficio ativado na COMPRA DO CARTÃO cria um beneficio
	            if($beneficiosCampanha[$i]['IDTPATIVABENEFICIO'] == '2'){
	            	array_push($beneficiosCompra, $beneficiosCampanha[$i]);
	            }else if($beneficiosCampanha[$i]['IDTPATIVABENEFICIO'] == '1'){
                    $beneficiosCampanha[$i]['QTBENEFICIOCAMP'] = 0;
                    array_push($beneficiosCompra, $beneficiosCampanha[$i]);
                }
	        }
	        for ($CDIDCONSUMID = $params['CARTAOINI']; $CDIDCONSUMID <= $params['CARTAOFIM']; ) {
	        	//dados consumidor
	        	$CDIDCONSUMID =	str_pad($CDIDCONSUMID, 9, "0", STR_PAD_LEFT);
	        	$CDIDCONSUMIDused =  self::criaDigitoVerificador($CDIDCONSUMID);
	        	$CDCONSUMIDOR = 'DOA'.str_pad($CDIDCONSUMIDused, 18, "0", STR_PAD_LEFT);
	            /* Definir campos não-nulos. */
	            $idatuconsumi = 'S';
	            $idsitconsumi = '1'; //ativo
	            $idconsumidor = 'C';
	            $idtpvendacons = '7';
	            $idimpcpfcupom = 'S';
	            $idcadconflibcon = 'N';
	            $idperconsprodex = 'S';
	            $idtpselmanha = 'N';
	            $idtpsealmoco = 'N';
	            $idtpseltarde = 'N';
	            $idcrachamestre = 'N';
	            $dadosCliente = self::buscaParamGeralFidelidade($params['NRORG']);
	            if($dadosCliente){
	            	$CDCLIENTE = $dadosCliente[0]['CDCLIENTE'];
	            	$CDCCUSCLIE = $dadosCliente[0]["CDCCUSCLIEFD"];
	            }else{
	            	$retorno = array('Contador' => $cont,
       				'Cartoes' => []);
	       			return $retorno;
	            }
	            /* Prepara os dados para o INSERT. */
	            $paramsAux = array(
	                ':CDCLIENTE'       => $CDCLIENTE,
	                ':CDCONSUMIDOR'    => $CDCONSUMIDOR,
	                ':CDIDCONSUMID'    => $CDIDCONSUMIDused,
	                ':NMCONSUMIDOR'    => "Cartão Concedido",
	                ':IDSITCONSUMI'    => $idsitconsumi,
	                ':CDCCUSCLIE'      => $CDCCUSCLIE,
	                ':IDCONSUMIDOR'    => $idconsumidor,
	                ':IDATUCONSUMI'    => $idatuconsumi,
	                ':IDTPVENDACONS'   => $idtpvendacons,
	                ':IDIMPCPFCUPOM'   => $idimpcpfcupom,
	                ':IDCADCONFLIBCON' => $idcadconflibcon,
	                ':IDPERCONSPRODEX' => $idperconsprodex,
	                ':IDTPSELMANHA'    => $idtpselmanha,
	                ':IDTPSEALMOCO'    => $idtpsealmoco,
	                ':IDTPSELTARDE'    => $idtpseltarde,
	                ':IDCRACHAMESTRE'  => $idcrachamestre,
	                ':NRORG'           => $params['NRORG']
	            );


	           	// validar se o CDIDCONSUMID já existe
	           	//se ja existe da rollback e manda mensagem de erro
	            $consumer = self::buscaConsumidor($CDIDCONSUMIDused, $params['NRORG']);
	           	if (!empty($consumer)) {
	           		if($erro == 0){
	           			$arrAux[] = '';
	           			$arrAux[] = 'Os seguintes cartões já estão cadastrados: ';
	           			$arrAux[] = '';
	           		}
	           		$arrAux[] = $CDIDCONSUMIDused;
	           		$erro++;
	           	} else {
		            //cria o cartao (consumidor)
		            self::criaCartaoDoacao($paramsAux);
		            //cria o beneficio
		            foreach ($beneficiosCompra as $beneficioCompra) {
			    	    //busca em uma query
				        $IDTPDESBENECONS = $beneficioCompra['IDTPBENEFICIO']; //query beneficio
				        $NRSEQBENEFICIO  = $beneficioCompra['NRSEQBENEFICIO'];//query
				        $VRDESBENECONS   = $beneficioCompra['VRBENEFICIO'];   //query
				        $NRSEQPRODCAMP   = $beneficioCompra['NRSEQPRODCAMP'];  //query
				        $CDOPERINCLUSAO  = $this->environment->getCdOperador();
				        $CDOPERULTATU    = null;
				        $NRORGINCLUSAO   = $this->environment->getNrOrgTrab();
				        $NRORGULTATU     = null;
				        $QTDEBENEFICIO   = $beneficioCompra['QTBENEFICIOCAMP'];
				        $IDIMPTMOVCTR    = 'N';
						//DTINIVALBENECONS = "hoje"
			            $NRDIASVALBENECAMP = $beneficioCompra['NRDIASVALBENECAMP'];

			            $DTFIMVALBENECONS   = Utilities\Util::nextNDay($NRDIASVALBENECAMP);
			            $DTFIMVALBENECONS   = str_replace("-","/",$DTFIMVALBENECONS);
			            $paramsDoacao = array(			
							':CDCAMPANHA' 		=> $params['CDCAMPANHA'], 
							':CDCLIENTE' 		=> $CDCLIENTE, 
							':CDCONSUMIDOR' 	=> $CDCONSUMIDOR, 
							':DTFIMVALBENECONS' => $DTFIMVALBENECONS,			
							':IDTPDESBENECONS' 	=> $IDTPDESBENECONS, 
							':NRSEQBENEFICIO' 	=> $NRSEQBENEFICIO, 
							':QTDEBENEFICIO' 	=> $QTDEBENEFICIO, 
							':VRDESBENECONS' 	=> $VRDESBENECONS, 
							':NRSEQPRODCAMP' 	=> $params['NRSEQPRODCAMP'], 
							':NRORG' 			=> $params['NRORG'], 
							':NRORGINCLUSAO' 	=> $NRORGINCLUSAO, 
							':CDOPERINCLUSAO' 	=> $CDOPERINCLUSAO, 
							':NRORGULTATU' 		=> $NRORGULTATU, 
							':CDOPERULTATU' 	=> $CDOPERULTATU,
							':IDIMPTMOVCTR' 	=> $IDIMPTMOVCTR
							
						);
			            self::insereBeneficioDoacao($paramsDoacao);
		            }
             		//acrescimo nos contadores
             		if($cont == 0){
             		$arrCad[0] = 'Os seguintes cartões foram cadastrados: ';
             		$arrCad[] = '';
             		}
             		$arrCad[] = $CDIDCONSUMIDused;
		            $cont++;
	       		}
		            $CDIDCONSUMID++;
	       	}
	       	$arr = array_merge($arrCad, $arrAux);
       		$retorno = array('Contador' => $cont,
       		'Cartoes' => $arr);
	       	return $retorno;
		} catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
		}
    }
    
    public function findProdutoBeneficio($params){	
    	return $this->entityManager->getConnection()->fetchAll(self::FIND_PRODUTO_GERA_BENEFICIO, $params);
    }
    
    public function findBeneficioDisponivel($params){
    	return $this->entityManager->getConnection()->fetchAll(self::FIND_BENEFICIO_DISPONIVEL, $params);
    }
    
    public function findBeneficioUtilizados($params){
    	return $this->entityManager->getConnection()->fetchAll(self::FIND_BENEFICIO_UTILIZADOS, $params);
    }
    
    const FIND_PRODUTO_GERA_BENEFICIO = "
    	SELECT DISTINCT BC.CDCAMPANHA, BC.NRSEQBENEFICIO, BC.NRSEQPRODCAMP, BC.QTBENEFICIOCAMP, BF.IDTPBENEFICIO,
			   (CASE BF.IDTPBENEFICIO  WHEN 'P' THEN 'Percentual' WHEN 'V' THEN 'Valor' END) AS NMTPBENEFICIO,
			   BF.VRBENEFICIO, BC.NRDIASVALBENECAMP, BF.IDTPATIVABENEFICIO, PC.CDPRODUTO, BC.NRORG, BF.DSBENEFICIO, P.NMPRODUTO
		  FROM BENEFICIOCAMP BC
		  JOIN BENEFICIOFOS BF
			ON BC.NRSEQBENEFICIO = BF.NRSEQBENEFICIO
		   AND BC.NRORG = BF.NRORG
		  JOIN PRODCAMPANHA PC
		    ON BC.CDCAMPANHA = PC.CDCAMPANHA
		   AND BC.NRSEQPRODCAMP = PC.NRSEQPRODCAMP
		   AND BC.NRORG = PC.NRORG
		  JOIN PRODUTO P
		    ON PC.CDPRODUTO = P.CDPRODUTO
		   AND PC.NRORG = P.NRORG
		 WHERE BC.NRORG = :NRORG
		   AND BC.CDCAMPANHA = :CDCAMPANHA
		   AND BF.IDTPATIVABENEFICIO = '2'
		 ORDER BY BC.NRSEQBENEFICIO, BC.NRSEQPRODCAMP
	";
									     
	const FIND_BENEFICIO_DISPONIVEL = "SELECT DISTINCT BC.QTBENEFICIOCAMP, BC.NRORG, BF.DSBENEFICIO, MV.VRDESBENECONS, MV.DTINIVALBENECONS, MV.DTFIMVALBENECONS
										    FROM MOVBENEFICIOCTR MV
										    JOIN BENEFICIOFOS BF
										      ON MV.NRSEQBENEFICIO = BF.NRSEQBENEFICIO
										     AND MV.NRORG = BF.NRORG
										    JOIN BENEFICIOCAMP BC
										      ON MV.NRORG = BC.NRORG
										      AND MV.NRSEQBENEFICIO = BC.NRSEQBENEFICIO
										      AND MV.CDCAMPANHA = BC.CDCAMPANHA
										      AND MV.NRSEQPRODCAMP = BC.NRSEQPRODCAMP
										   WHERE MV.NRORG = :NRORG
										     AND MV.CDCLIENTE = :CDCLIENTE
										     AND MV.CDCONSUMIDOR = :CDCONSUMIDOR
										     ORDER BY MV.DTINIVALBENECONS";
										     
	const FIND_BENEFICIO_UTILIZADOS = "SELECT DISTINCT BC.NRORG, BF.DSBENEFICIO, MV.CDFILIAL||' - '||F.NMFILIAL AS LABELFILIAL, MV.CDCAIXA, MV.DTUTBENECONS, MV.NRSEQVENDA
										    FROM MOVBENEFICIOCONS MV
										    JOIN BENEFICIOFOS BF
										      ON MV.NRSEQBENEFICIO = BF.NRSEQBENEFICIO
										     AND MV.NRORG = BF.NRORG
										    JOIN BENEFICIOCAMP BC
										      ON MV.NRORG = BC.NRORG
										      AND MV.NRSEQBENEFICIO = BC.NRSEQBENEFICIO
										      AND MV.NRSEQPRODCAMP = BC.NRSEQPRODCAMP
										      AND MV.CDCAMPANHA = BC.CDCAMPANHA
										    JOIN FILIAL F
										      ON MV.NRORG = F.NRORG
										   WHERE MV.NRORG = :NRORG
										     AND F.CDFILIAL = MV.CDFILIAL
										     AND MV.CDCLIENTE = :CDCLIENTE
										     AND MV.CDCONSUMIDOR = :CDCONSUMIDOR
										     ORDER BY BF.DSBENEFICIO";
    
}